#! /usr/bin/env python
###############################################################################
#
#  Script name                  : ChassisUpdate.py
#
#  Dependencies                 : ComDefs.py
#                    
#  Objective                    : Startup script for BladeCenterUpdates
#                    
#  Written by                   : John R. Hicks
#                                 jrhicks@us.ibm.com
#                                 Robert K. Christner
#                                 christne@us.ibm.com
#                     
#  Change History               : who -   when   - what
#                                 JRH - 07/12/04 - Initial revision.
#                     		      JRH - 08/03/04 - Added ability to read 
#                                                  options from response file,
#						                           and moved switch discovery
#                                                  to occur before flashing
#						                           the MM.
#                                 RKC - 09/01/04 - Added final status messages 
#                                 RKC - 09/27/04 - Added rebootMMweb, connectMM
#                                                  and handleReboot
#                                 RKC - 09/29/04 - Major rework to FlashMM logic
#                                                  and added verifyCodeLvl,
#                                                  stopIfDhcp, and extraReboot
#  
#  Copyright IBM Corporation, 2004
#
###############################################################################

import sys, socket, time, getopt, string, os.path, urllib
try:
    from BladeCenterUpdates.Common import ComDefs, ComFuncs
    from BladeCenterUpdates.Common.Logger import Logger
    from BladeCenterUpdates.Common.MMCommunications import MMCom
    from BladeCenterUpdates.MM import MMInfo, MMFlash
    from BladeCenterUpdates.IOModules import IOInfo, IBMSMFlash, NortelSMFlash
except ImportError:
    print "Error! Unable to locate the BladeCenterUpdates package.  The most likely cause"
    print "of this error is an improperly set PYTHONPATH environment variable.  Please set"
    print "the PYTHONPATH environment variable to the directory above the location of this"
    print "script."
    sys.exit(1)
except KeyboardInterrupt:
    sys.exit(1)
            
params = ""
logger = ""
mmCom = ""
isTest = ComDefs.FALSE
hardwareRev = 0
IOModuleIsInstalled = ""
IOModuleIP = ""
IOModuleType = ""
IOModuleLevel = ""
IOModuleCanPing = ComDefs.FALSE
numIObays = 4
noExtraReboot = ComDefs.FALSE

class outputStrings:
    #Pass = '\x1b[0m\x1b[1;32mSuccess\x1b[0m'
    #Fail = '\x1b[0m\x1b[1;31mFail   \x1b[0m'
    Pass    = 'Success                '
    Fail    = 'Fail                   '
    Warning = 'Success (with warnings)'
    IBM    = '(IBM ESM)    '
    Nortel = '(Nortel)     '
    unsup  = '(unsupported)'

class componentStatus:
    finalStatus = 'N/A                    '
    component = '(empty)      '
    
class updateStatus:
    str = outputStrings()
    MM = componentStatus()
    io = []
    # there is no I/O bay 0
    for i in xrange(numIObays + 1):
        io.append(componentStatus())

status = updateStatus()


def print_matrix():

    global status

    print
    print
    print "+---------------------------------------------------+"
    print "|        Component        |         Status          |"
    print "|=========================|=========================|"
    print "| Management Module       | %s |" % status.MM.finalStatus 
    print "|-------------------------|-------------------------|"
    for a in xrange(numIObays):
        i = a + 1
        print "| I/O bay %s %s | %s |" % (i, status.io[i].component, status.io[i].finalStatus) 
    print "+---------------------------------------------------+"
    print


def print_start():

    print
    print "+------------------------------------------------------+"
    print "| %s |" % ComDefs.PRODUCT
    print "|------------------------------------------------------|"
    print "| Version %s        Copyright IBM Corporation, 2004 |" % ComDefs.VERSION
    print "+======================================================+"
    print


# The MM firmware packets and what they are
MM_FIRMWARE_PACKETS = {"Boot ROM":"CNETBRUS.PKT", "Main Application":"CNETMNUS.PKT",
                       "Remote Control":"CNETRGUS.PKT"}
                       

def FlashIOModules(installedModules):
    """
    	Loops through all the I/O Module bays and updates
    	the firmware for each installed module.
    """
    global logger, params, status

    for module in installedModules:
        if module.manufacturer == ComDefs.IBM_SWITCH_ID:
            status.io[module.moduleBay].component = status.str.IBM
            IBMSMFlash.params = params
            IBMSMFlash.logger = logger
            print
            logger.log("Updating IBM ESM in bay %s" % module.moduleBay, ComDefs.TRUE)
            rc = IBMSMFlash.flash_switch(module.ipAddress, module.moduleBay)
            if rc == ComDefs.SUCCESS:
                status.io[module.moduleBay].finalStatus = status.str.Pass
                logger.log("Success! IBM ESM update complete.", ComDefs.TRUE)
            elif rc == ComDefs.WARNING:
                status.io[module.moduleBay].finalStatus = status.str.Warning
                logger.log("Success! IBM ESM update complete (with warnings).", ComDefs.TRUE)
            else:
                status.io[module.moduleBay].finalStatus = status.str.Fail
                logger.log(ComDefs.GetLastError(rc), ComDefs.TRUE)
                
        elif module.manufacturer == ComDefs.NORTEL_SWITCH_ID:
            status.io[module.moduleBay].component = status.str.Nortel
            NortelSMFlash.params = params
            NortelSMFlash.logger = logger
            print
            logger.log("Updating Nortel switch in bay %s" % module.moduleBay, ComDefs.TRUE)
            rc = NortelSMFlash.flash_switch(module.ipAddress, module.moduleBay)
            if rc != ComDefs.SUCCESS:
                status.io[module.moduleBay].finalStatus = status.str.Fail
                logger.log(ComDefs.GetLastError(rc), ComDefs.TRUE)
            else:
                status.io[module.moduleBay].finalStatus = status.str.Pass
                logger.log("Success! Nortel switch update complete.", ComDefs.TRUE)
             
                

def FlashMM():
    """
    	Flashes the Management Module firmware by calling the MMFlash.py 
    	script.    

        Returns:
            ComDefs.SUCCESS to continue with IO
            anything else to exit
    """
    global mmCom, logger, params, status, noExtraReboot
    
    MMInfo.logger = logger
    MMInfo.mmCom = mmCom
    
    # Determine the firmware level of the update images
    packetFile = os.path.join(params.mmLocation, string.upper(MM_FIRMWARE_PACKETS["Boot ROM"]))
    pktCodeLvl = getPacketRev(packetFile)
    logger.log("Management Module firmware update file level: %d" % pktCodeLvl)
    if pktCodeLvl == 0:
        logger.log(ComDefs.GetLastError(ComDefs.MM_INVALID_PACKET_FILE), ComDefs.TRUE)
        status.MM.finalStatus = status.str.Fail
        return ComDefs.SUCCESS

    # Determine the firmware level of the primary Management Module
    rc = MMInfo.getBuildIDs(1)[4:-1]
    if rc == "":
        logger.log("Error getting Management Module firmware level", ComDefs.TRUE)
        logger.log(ComDefs.GetLastError(MMInfo.lastError), ComDefs.TRUE)
        return ComDefs.GENERAL_UPDATE_FAILURE

    mmCodeLvl = int(rc)
    logger.log("Management Module firmware code level: %d" % mmCodeLvl)

    if pktCodeLvl < mmCodeLvl:
        logger.log(ComDefs.NO_BACKLEVEL, ComDefs.TRUE)
        status.MM.finalStatus = status.str.Fail
        return ComDefs.SUCCESS

    # MM code level is less than 52
    if mmCodeLvl < ComDefs.REDUNDANT_MM_CODELEVEL:
        # For now we will go ahead with this update.  If the user has redundant
        # Management Modules below 52 that are configured with DHCP, the first
        # MM will get updated, and unless the second comes up with the same IP
        # it will not be found, and the flash will fail.
        #if MMInfo.IsMMDhcp():
        #    if stopIfDhcp():
        #        return ComDefs.FALSE
        logger.log("Updating Management Module from Level %d to Level %d" % (mmCodeLvl, pktCodeLvl))
        rc = PerformMMFlash(ComDefs.TRUE)
        if rc == ComDefs.FALSE:
            return ComDefs.SUCCESS

        # Reconnect to the Management Module, retrying/rebooting if necessary
        rc = connectMM(ComDefs.TRUE)
        if rc != ComDefs.SUCCESS:
            logger.log(ComDefs.GetLastError(rc), ComDefs.TRUE)
            status.MM.finalStatus = status.str.Fail
            return ComDefs.SUCCESS

        # query for MM firmware level
        rc = MMInfo.getBuildIDs(1)[4:-1]
        if rc == "":
            logger.log("Error getting Management Module firmware level", ComDefs.TRUE)
            logger.log(ComDefs.GetLastError(MMInfo.lastError), ComDefs.TRUE)
            return ComDefs.GENERAL_UPDATE_FAILURE
        newCodeLvl = int(rc)
        logger.log("Management Module firmware code level: %d" % newCodeLvl)

        # see if we have a redundant MM or not
        if newCodeLvl == pktCodeLvl:
            logger.log(ComDefs.MM_LEVEL_VERIFIED)
            logger.log(ComDefs.MM_FLASH_SUCCESS, ComDefs.TRUE)
            extraReboot()
            # wait a bit longer for the I/O to come up
            logger.log(ComDefs.IO_WAIT_MESSAGE % ComDefs.IO_WAIT_TIME, ComDefs.TRUE)
            time.sleep(ComDefs.IO_WAIT_TIME)
            status.MM.finalStatus = status.str.Pass
            return ComDefs.SUCCESS
        elif newCodeLvl > pktCodeLvl:
            logger.log(ComDefs.NO_BACKLEVEL, ComDefs.TRUE)
            # wait a bit longer for the I/O to come up
            logger.log(ComDefs.IO_WAIT_MESSAGE % ComDefs.IO_WAIT_TIME, ComDefs.TRUE)
            time.sleep(ComDefs.IO_WAIT_TIME)
            status.MM.finalStatus = status.str.Fail
            return ComDefs.SUCCESS

        # we have a redundant MM, flash it
        logger.log("Updating Management Module from Level %d to Level %d" % (newCodeLvl, pktCodeLvl))
        rc = PerformMMFlash(ComDefs.TRUE)
        if rc == ComDefs.FALSE:
            return ComDefs.SUCCESS
    # MM code level is greater than or equal to 52
    else:
        rMMBay = MMInfo.IsRedundantMMInstalled()
        # We have a redundant MM
        if rMMBay > 0:
            #logger.log("Redundant Management Module Installed", ComDefs.TRUE)
            # MM code level is less than 57
            if mmCodeLvl < ComDefs.REDUNDANT_MM_CODELEVEL:
                # The MM is configured with DHCP
                if MMInfo.IsMMDhcp():
                    if stopIfDhcp():
                        return ComDefs.GENERAL_UPDATE_ERROR
            # The MM has a static IP
            # get redundant MM firmware level
            rc = MMInfo.getRedundantBuildIDs(1)[4:-1]
            if rc == "":
                logger.log("Error getting Redundant Management Module firmware level")
                if MMInfo.lastError == ComDefs.MM_SEND_FAILURE:
                    logger.log("Assume redundant firmware level is less than 52 (i.e. 27)")
                    rc = "27"
                else:
                    logger.log(ComDefs.GetLastError(MMInfo.lastError))
                    return ComDefs.GENERAL_UPDATE_FAILURE

            redCodeLvl = int(rc)
            logger.log("Redundant Management Module firmware code level: %d" % redCodeLvl)
            # redundant is less than 57
            if redCodeLvl < ComDefs.SUPPORTED_MM_CODELEVEL:
                logger.log("Updating Management Module from Level %d to Level %d" % (mmCodeLvl, pktCodeLvl))
                rc = PerformMMFlash(ComDefs.TRUE)
                if rc == ComDefs.FALSE:
                    return ComDefs.SUCCESS
                # Reconnect to the Management Module, retrying/rebooting if necessary
                rc = connectMM(ComDefs.TRUE)
                if rc != ComDefs.SUCCESS:
                    logger.log(ComDefs.GetLastError(rc), ComDefs.TRUE)
                    return ComDefs.SUCCESS
                rc = MMInfo.getBuildIDs(1)[4:-1]
                if rc == "":
                    logger.log("Error getting Management Module firmware level", ComDefs.TRUE)
                    logger.log(ComDefs.GetLastError(MMInfo.lastError), ComDefs.TRUE)
                    return ComDefs.GENERAL_UPDATE_FAILURE

                mmCodeLvl = int(rc)
                logger.log("Updating Management Module from Level %d to Level %d" % (mmCodeLvl, pktCodeLvl))
                rc = PerformMMFlash(ComDefs.TRUE)
                if rc == ComDefs.FALSE:
                    return ComDefs.SUCCESS
            # redundant is greater than or equal to 57
            else:
                logger.log("Updating Management Module from Level %d to Level %d" % (mmCodeLvl, pktCodeLvl))
                noExtraReboot = ComDefs.TRUE
                rc = PerformMMFlash()
                if rc == ComDefs.FALSE:
                    return ComDefs.SUCCESS
        # We do not have a redundant MM
        else:
            logger.log("Updating Management Module from Level %d to Level %d" % (mmCodeLvl, pktCodeLvl))
            rc = PerformMMFlash()
            if rc == ComDefs.FALSE:
                return ComDefs.SUCCESS

    # reboot one extra time just to be sure everything is settled
    if noExtraReboot == ComDefs.FALSE:
        extraReboot()
    # verify the firmware is at the desired level
    if verifyCodeLvl(pktCodeLvl):
        logger.log(ComDefs.MM_FLASH_SUCCESS, ComDefs.TRUE)
        status.MM.finalStatus = status.str.Pass
    else:
        status.MM.finalStatus = status.str.Fail

    # wait a bit longer for the I/O to come up
    logger.log(ComDefs.IO_WAIT_MESSAGE % ComDefs.IO_WAIT_TIME, ComDefs.TRUE)
    time.sleep(ComDefs.IO_WAIT_TIME)
    return ComDefs.SUCCESS
    
def PerformMMFlash(failover=ComDefs.FALSE):
    """
        Returns:
            ComDefs.FALSE - failed
            ComDefs.TRUE - success
    """

    global status, logger, mmCom, noExtraReboot

    MMFlash.mmCom = mmCom
    MMFlash.logger = logger


    for prettyName, packetName in MM_FIRMWARE_PACKETS.iteritems():
        packetFile = os.path.join(params.mmLocation, string.upper(packetName))
        rc = MMFlash.updateFirmware(packetFile)
        
        logger.log("MMFlash return code: %d" % rc)
        
        if rc != ComDefs.SUCCESS:
            flashSuccess = ComDefs.FALSE
            if rc == ComDefs.MM_PACKET_FILE_NOT_FOUND:
                logger.log("Flash failed:  Packet file %s not found." % packetName)
            elif ((rc == ComDefs.MM_INVALID_PACKET_FILE) |
                 (rc == ComDefs.MM_INVALID_PACKET_FILE_HEADER)):
                logger.log("\nError flashing the MM %s!\nThe packet file %s is not valid!" % (prettyName, packetName))
            else:
                logger.log("\nThere was an unknown error flashing the MM %s!" % prettyName)
        else:
            logger.log("Flashing the MM %s, please wait." % prettyName, ComDefs.TRUE)
            counter = 0
            while MMFlash.getUpdateStatus()[0] == chr(255):
                if counter % 40 == 0:
                    sys.stdout.write('.')
                    sys.stdout.flush()
                counter += 1
            print
            logger.log("Successfully flashed the MM %s." % prettyName, ComDefs.TRUE)
            flashSuccess = ComDefs.TRUE
                        
    if flashSuccess:
        logger.log("Completed MM Flash.  Rebooting MM", ComDefs.TRUE)

        MMFlash.rebootMM(failover)

        # wait for the MM to come back up
        if noExtraReboot == ComDefs.TRUE:
            logger.log("Waiting %s seconds for the Management Module to reboot." % ComDefs.MM_REBOOT_TIME, ComDefs.TRUE)
            time.sleep(ComDefs.MM_REBOOT_TIME)
        else:
            logger.log("Waiting %s seconds for the Management Module to reboot." % ComDefs.MM_QUICK_REBOOT_TIME, ComDefs.TRUE)
            time.sleep(ComDefs.MM_QUICK_REBOOT_TIME)
        return ComDefs.TRUE
    else:
        status.MM.finalStatus = status.str.Fail
        logger.log(ComDefs.GetLastError(rc), ComDefs.TRUE)
        return ComDefs.FALSE
        

def stopIfDhcp():
    """
    Returns:
        ComDefs.FALSE - if you should continue
        ComDefs.TRUE - to exit
    """
    global params, logger, status

    # If we're at code level 57 or less and using DHCP, we can't flash the 
    # redundant MM, so at this point we ask the user if they want to continue
    # flashing the I/O modules.  If the --continueIO switch is specified 
    # by the user, we bypass the prompt and just flash the switch modules.
    
    if params.continueIO == ComDefs.FALSE:
        #FIXME - pretty this up
        logger.log("Managemnet Module code level below minimum supported level for DHCP.", ComDefs.TRUE)
        logger.log("Cannot update Management Module.", ComDefs.TRUE)
        print "**To skip the prompt and automatically continue,"
        print "**please specify the continueIO key in the response file.\n"
        print "Would you like to continue flashing the I/O modules? (Y/N)"
        response = sys.stdin.read(1)
        #FIXME - do slightly better error checking here
        if string.upper(response) == "Y":
            status.MM.finalStatus = status.str.Fail
            return ComDefs.FALSE
        else:
            logger.log("User exit requested.", ComDefs.TRUE)
            return ComDefs.TRUE
    else:
        return ComDefs.FALSE

def verifyCodeLvl(pktCodeLvl):
    """
        Verifies that the current MM Code Level matches the Level of the update packet files.
        Logs onto the MM, queries, and leaves the connection open.

        Returns:
            ComDefs.TRUE - verified
            ComDefs.FALSE - unverified
    """
    global logger, mmCom

    MMInfo.logger = logger
    MMInfo.mmCom = mmCom

    logger.log("Verifying the Management Module Code Level.", ComDefs.TRUE)
    # Reconnect to the Management Module
    rc = connectMM()
    if rc != ComDefs.SUCCESS:
        logger.log("Could not verify the firmware level of the Management Module.", ComDefs.TRUE)
        logger.log(ComDefs.GetLastError(rc), ComDefs.TRUE)
        return ComDefs.FALSE

    rc = MMInfo.getBuildIDs(1)[4:-1]
    if rc == "":
        logger.log("Error getting Management Module firmware level", ComDefs.TRUE)
        logger.log(ComDefs.GetLastError(MMInfo.lastError), ComDefs.TRUE)
        return ComDefs.FALSE

    mmCodeLvl = int(rc)
    if mmCodeLvl == pktCodeLvl:
        logger.log(ComDefs.MM_LEVEL_VERIFIED)
        return ComDefs.TRUE

    logger.log("The firmware level of the Management Module (%d)" % mmCodeLvl, ComDefs.TRUE)
    logger.log("is different than the update file (%d)." % pktCodeLvl, ComDefs.TRUE)
    return ComDefs.FALSE

def extraReboot(failover=ComDefs.FALSE):
    """
        Initiates a connect to the MM and reboots it via dot commands
    """
    global logger, mmCom

    MMFlash.logger = logger
    MMFlash.mmCom = mmCom
    
    logger.log("Rebooting the Management Module to ensure all updates are settled.", ComDefs.TRUE)
    rc = connectMM()
    if rc != ComDefs.SUCCESS:
        logger.log(ComDefs.GetLastError(rc), ComDefs.TRUE)
        return rc
    MMFlash.rebootMM(failover)
    logger.log("Waiting %s seconds for the Management Module to reboot." % ComDefs.MM_REBOOT_TIME, ComDefs.TRUE)
    time.sleep(ComDefs.MM_REBOOT_TIME)
    return ComDefs.SUCCESS


def usage():
    """ Displays usage instructions to the user """
    # Terminal ends ......................................................................V
    print
    print "Usage: ChassisUpdate.py [options]"
    print
    print "Options:"
    #print "  --mmipaddr=IPADDR        The hostname or IP address of the Management Module"
    #print "  --mmuser=USERNAME        The administrator username used to log into the"
    #print "                           Management Module"
    #print "  --mmpass=PASSWORD        The password of the administrator username used to"
    #print "                           log into the Management Module"
    #print "  --mmFileLocation=PATH    The full path to the directory containing the"
    #print "                           Management Module update files"
    #print "  --io1user=USERNAME       The username used to log into the I/O module in bay 1"
    #print "  --io1pass=PASSWORD       The password used to log into the I/O module in bay 1"
    #print "  --io1Filename1=FILE      The path and name of the first (OS) firmware update"
    #print "                           file on the TFTP server for the I/O module in bay 1"
    #print "  --io1Filename2=FILE      The path and name of the second (boot) firmware"
    #print "                           update file on the TFTP server for the I/O module in"
    #print "                           Bay 1"
    #print "  --io2user=USERNAME       The username used to log into the I/O module in bay 2"
    #print "  --io2pass=PASSWORD       The password used to log into the I/O module in bay 2"
    #print "  --io2Filename1=FILE      The path and name of the first (OS) firmware update"
    #print "                           file on the TFTP server for the I/O module in bay 2"
    #print "  --io2Filename2=FILE      The path and name of the second (boot) firmware"
    #print "                           update file on the TFTP server for the I/O module in"
    #print "                           Bay 2"
    #print "  --io3user=USERNAME       The username used to log into the I/O module in bay 3"
    #print "  --io3pass=PASSWORD       The password used to log into the I/O module in bay 3"
    #print "  --io3Filename1=FILE      The path and name of the first (OS) firmware update"
    #print "                           file on the TFTP server for the I/O module in bay 3"
    #print "  --io3Filename2=FILE      The path and name of the second (boot) firmware"
    #print "                           update file on the TFTP server for the I/O module in"
    #print "                           Bay 3"
    #print "  --io4user=USERNAME       The username used to log into the I/O module in bay 4"
    #print "  --io4pass=PASSWORD       The password used to log into the I/O module in bay 4"
    #print "  --io4Filename1=FILE      The path and name of the first (OS) firmware update"
    #print "                           file on the TFTP server for the I/O module in bay 4"
    #print "  --io4Filename2=FILE      The path and name of the second (boot) firmware"
    #print "                           update file on the TFTP server for the I/O module in"
    #print "                           Bay 4"
    #print "  --tftpipaddr=IPADDR      The IP address of the TFTP server"
    #print "  --continueIO=TRUE        Automatically update I/O modules if Management Module"
    #print "                           update fails"
    #print "  --forceMMreboot=TRUE     Force a reboot of the Management Module after three"
    #print "                           unsuccessfull login attempts"
    print "  -r, --responseFile=FILE  The path and name of the response file.  If this"
    print "                           option is not provided, the default response file"
    print "                           ./BladeCenterUpdates.rsp is used"
    print "  -h, --help, -?           Display this (help) message"
    print "  -v, --verbose            Increase the debug output level by one"
    print "  -V, --version            Display the ChassisUpdate version number and exit"
    

def parse_args(opts):
    global params, isTest
    for o, a in opts:
        if o in ("-?", "-h", "--help"):
            usage()
            sys.exit()
        elif o in ("-t", "--test", "test"):
            isTest = ComDefs.TRUE
        elif o in ("-v", "--verbose"):
            params.debugLevel += 1
        elif o in ("-V", "--version"):
            print ComDefs.PRODUCT + ' ' + ComDefs.VERSION
            sys.exit()
        elif o in ("mmipaddr", "--mmipaddr"):
            params.mmIPAddress = a
        elif o in ("mmuser",  "--mmuser"):
            params.mmUserName = a
        elif o in ("mmpass", "--mmpass"):
            params.mmPassword = a
        elif o in ("mmFileLocation", "--mmFileLocation"):
            params.mmLocation = a
        elif o in ("io1user", "--io1user"):
            params.io1UserName = a
        elif o in ("io1pass", "--io1pass"):
            params.io1Password = a
        elif o in ("io2user", "--io2user"):
            params.io2UserName = a
        elif o in ("io2pass", "--io2pass"):
            params.io2Password = a
        elif o in ("io3user", "--io3user"):
            params.io3UserName = a
        elif o in ("io3pass" "--io3pass"):
            params.io3Password = a
        elif o in ("io4user", "--io4user"):
            params.io4UserName = a
        elif o in ("io4pass", "--io4pass"):
            params.io4Password =a
        elif o in ("io1Filename1", "--io1Filename1"):
            params.io1Filename1 = a
        elif o in ("io1Filename2", "--io1Filename2"):
            params.io1Filename2 = a
        elif o in ("io2Filename1", "--io2Filename1"):
            params.io2Filename1 = a
        elif o in ("io2Filename2", "--io2Filename2"):
            params.io2Filename2 = a
        elif o in ("io3Filename1", "--io3Filename1"):
            params.io3Filename1 = a
        elif o in ("io3Filename2", "--io3Filename2"):
            params.io3Filename2 = a
        elif o in ("io4Filename1", "--io4Filename1"):
            params.io4Filename1 = a
        elif o in ("io4Filename2", "--io4Filename2"):
            params.io4Filename2 = a
        elif o in ("tftpipaddr", "--tftpipaddr"):
            params.tftpIPAddr = a
        elif o in ("continueIO", "--continueIO"):
            if string.upper(a) == "TRUE":
                params.continueIO = ComDefs.TRUE
        elif o in ("forceMMreboot", "--forceMMreboot"):
            if string.upper(a) == "TRUE":
                params.forceMMreboot = ComDefs.TRUE

    return ComDefs.SUCCESS
    
def validate_args():
    """ Parse provided arguments to determine if all
    	required arguments have been provided.  If not,
    	print usage message and quit.
    """
    global params, isTest
    
#    if isTest == ComDefs.TRUE:
#        print "mmIPAddress: %s" % params.mmIPAddress
#        print "mmUserName: %s" % params.mmUserName
#        print "mmPassword: %s" % params.mmPassword
#        print "io1UserName: %s" % params.io1UserName
#        print "io1Password: %s" % params.io1Password
#        print "io1Filename1: %s" % params.io1Filename1
#        print "io1Filename2: %s" % params.io1Filename2
        
    
    if (params.mmIPAddress == "") or (params.mmUserName == "") or (params.mmPassword == ""):
        # Print usage and quit
        usage()
        return ComDefs.GENERAL_UPDATE_ERROR

    return ComDefs.SUCCESS
        
def parse_responses():
    """ Read responses contained in response file, and populate
    	parameters with those responses, if they exist.
  	"""                
    try:
        input = open(params.responseFile, 'r')
    except IOError:
        print "\nCannot open response file %s for reading." % params.responseFile
        print "Using command line arguements instead.\n"
        return ComDefs.SUCCESS

    opts = []
    
    if input == None:
        print "Error! Response file is empty."
        return ComDefs.GENERAL_UPDATE_ERROR
        
    for line in input:
        line = line.strip(' ')
        if line[0] == '#' or line[0] == "\n" or line[0] == ' ':
            continue
        else:
            a = line.find('\n')
            if a > 0:
                line = line.strip('\n')
            opt = line.split('=')
            opts.append(opt)

    # parse response file arguements
    rc = parse_args(opts)
    if rc != ComDefs.SUCCESS:
        return rc
    
    input.close()
    return ComDefs.SUCCESS

def rebootMMweb(doFailOver = ComDefs.FALSE):
    """ Reboot the MM via the web interface
    Specify ComDefs.TRUE for doFailOVer if you want the MM to fail over to the redundant MM
    This is only supported on MM firmware levels at 52 or higher
    
    Return Codes:
        ComDefs.SUCCESS
        ComDefs.MM_SOCKET_ERR
    """
    global logger, params

    rc = ComDefs.SUCCESS
    if doFailOver:
        # Send the reboot command and let it fail over if there is a redundant MM
        try:
            u = urllib.urlopen("http://%s:%s@%s/private/failover_mm" % (params.mmUserName, params.mmPassword, params.mmIPAddress))
            u.close()
        except:
            logger.log("Failover of Management Module failed.  Could not connect to web interface.")
            rc = ComDefs.MM_SOCKET_ERR
    else:
        # Send the reboot command and do not let it fail over if there is a redundant MM
        try:
            u = urllib.urlopen("http://%s:%s@%s/private/restart_asm" % (params.mmUserName, params.mmPassword, params.mmIPAddress))
            u.close()
        except:
            logger.log("Restart of Management Module failed.  Could not connect to web interface.")
            rc = ComDefs.MM_SOCKET_ERR

    return rc


def handleReboot():
    """
    Reboot the Management Module of the forceMMreboot parameter is passes,
    and handle calling connectMM again if reboot succeeds.
    """
    global params, logger

    if params.forceMMreboot:
        logger.log("Rebooting the Management Module.", ComDefs.TRUE)
        rc = rebootMMweb()
        if rc != ComDefs.SUCCESS:
            logger.log("Error! Reboot of the Management Module failed.", ComDefs.TRUE)
            return ComDefs.MM_SOCKET_ERR
        else:
            logger.log("Waiting %s seconds for the Management Module to reboot." % ComDefs.MM_REBOOT_TIME, ComDefs.TRUE)
            time.sleep(ComDefs.MM_REBOOT_TIME)
            return connectMM()
    else:
        print
        logger.log("To force a reboot of the Management Module in this situation specify the", ComDefs.TRUE)
        logger.log("forceMMreboot option in the response file.", ComDefs.TRUE)
        return ComDefs.MM_SOCKET_ERR


def connectMM(reboot=ComDefs.FALSE, maxRetries=3):
    """
    Try to connect to the Management Module.  If there is a socket error, 
    retry "maxRetries" times.  Then reboot once, and start the retry period
    again.
    """
    global params, logger, mmCom

    rc = mmCom.open(ComDefs.TRUE, params.mmIPAddress, 6090, 5, params.mmUserName, params.mmPassword)
    
    # Because applications (like Director) access the RSA, RSA2, and
    # MM on port 6090 (which the MM uses), we loop through the MM
    # connection 3 times if we can't access port 6090.  We quit after
    # attempt number "maxRetries".
    if rc == ComDefs.MM_SOCKET_ERR:
        mmAttempt = 1
        while mmAttempt <= maxRetries:
            logger.log("Socket error connecting to Management Module (%s).  Retry #%d" % (params.mmIPAddress, mmAttempt), ComDefs.TRUE)
            time.sleep(10)
            rc = mmCom.open(ComDefs.TRUE, params.mmIPAddress, 6090, 5, params.mmUserName, params.mmPassword)
            if rc != ComDefs.MM_SOCKET_ERR:
                break
            mmAttempt += 1
            
        if rc == ComDefs.MM_SOCKET_ERR and mmAttempt > maxRetries:
            logger.log("Maximum retries exceeded.  Unable to connect to the Management Module.", ComDefs.TRUE)
            logger.log("Verify network settings and the mmipaddr entry of the response file.", ComDefs.TRUE)
            if reboot:
                rc = handleReboot()

    # wait a few seconds before trying to access the MM
    time.sleep(5)

    return rc


def getPacketRev(packetFile):
    """
        Determines the firmware revision given the packet file
    """
    global logger

    version = 0
    
    try:
        packet = open(packetFile, 'rb')
    except IOError:
        logger.log("\nCannot open packet file %s for reading." % packetFile)
        return 0

    packet.seek(113);
    version = int(packet.read(2))
    packet.close()

    return version


def main():
    """ Main entry point """
    
    # Define using global variables
    global mmCom, params, logger, hardwareRev
    
    # Instanciate the class
    params = ComDefs.parameters()
 
    # Parse command line options with getopt()
    try:
        opts, args = getopt.getopt(sys.argv[1:], "?htVvr:",
            ["?", "help", "test", "verbose", "version", 
             "mmipaddr=", "mmuser=", "mmpass=", "mmFileLocation=", 
             "io1user=", "io1pass=", "io2user=", "io2pass=", 
             "io3user=", "io3pass=", "io4user=", "io4pass=", 
             "io1Filename1=", "io1Filename2=", "io2Filename1=", "io2Filename2=", 
             "io3Filename1=", "io3Filename2=", "io4Filename1=", "io4Filename2=",
             "tftpipaddr=", "continueIO=", "forceMMreboot=", "responseFile="])

    except getopt.GetoptError:
        # Print usage information and exit
        usage()
        return ComDefs.GENERAL_UPDATE_ERROR

    # Get the response file arguement
    for o, a in opts:
        if o in ("-r", "--responseFile"):
            params.responseFile = a 
            break
    # Handle the arguements from response file
    rc = parse_responses()
    if rc != ComDefs.SUCCESS:
        return rc
    # All response file args are overwritten by command line args
    # This is by design
    rc = parse_args(opts)
    if rc != ComDefs.SUCCESS:
        return rc
    rc = validate_args()
    if rc != ComDefs.SUCCESS:
        return rc

    # Initialize the Logger
    logger = Logger(params)

    # Print pretty start banner
    print_start()

    mmCom = MMCom(params.debugLevel)

    # Initialize dependent module logger objects
    MMInfo.logger = logger
    MMFlash.logger = logger
    MMCom.logger = logger
    mmCom.logger = logger
    
    # We have to pass the MMCommunications object on to
    # each of the modules we use to access the MM and I/O module.
    MMInfo.mmCom = mmCom
    MMFlash.mmCom = mmCom
    IOInfo.mmCom = mmCom

    # Connect to the Management Module, retrying/rebooting if necessary
    rc = connectMM(ComDefs.TRUE)
    if rc != ComDefs.SUCCESS:
        logger.log(ComDefs.GetLastError(rc), ComDefs.TRUE)
        return rc

    logger.log("Successfully logged into management module")
        
    hardwareRev = MMInfo.getChassisHardwareRev()
    if hardwareRev == -1:
        logger.log("Error getting hardware revision.", ComDefs.TRUE)
        logger.log(ComDefs.GetLastError(MMInfo.lastError), ComDefs.TRUE)
        return ComDefs.GENERAL_UPDATE_ERROR

    logger.log("Chassis hardware revision: %d" % hardwareRev)
        
    # If the hardware revision of the chassis midplane is less than 5,
    # display an error and exit.
    if (hardwareRev < ComDefs.SUPPORTED_MIDPLANE_LEVEL):
        message = "Hardware revision %d is not supported.  Must be at least revision %s." \
                   % (hardwareRev, ComDefs.SUPPORTED_MIDPLANE_LEVEL)
        logger.log(message, ComDefs.TRUE)
        return ComDefs.GENERAL_UPDATE_ERROR
    
    # Loop through all the I/O modules and gather their info.  We do this so that we
    # ensure that we can flash the I/O modules after rebooting the MM.
    
    # Create IOInfo.IOModuleInfo objects for each of the 4 switch bays.
    ioModules = []
    for a in xrange(numIObays):
        ioModules.append(IOInfo.IOModuleInfo(a + 1))
    
    installedModules = []
    for a in ioModules:
        if a.isInstalled == ComDefs.TRUE:
            status.io[a.moduleBay].component = status.str.unsup
            installedModules.append(a)
    
    # Flash the MM
    print "\nUpdating Management Module"
    print "--------------------------\n"
    rc = FlashMM()
    if rc != ComDefs.SUCCESS:
        return rc

    # Loop through all the I/O modules, gather their info, and flash them.
    print "\n\nUpdating I/O Module(s)"
    print "----------------------"
    FlashIOModules(installedModules)
    #if rc != ComDefs.SUCCESS:
    #    return rc

    mmCom.close()

    print_matrix()
    
# This get's invoked if they run this script directly
if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        ComFuncs.keyboardISR(logger)
