#! /usr/bin/env python
###############################################################################
#
#  Script name                  : MMInfo.py
#
#  Dependencies                 : ComFuncs.py
#                                 ComDefs.py
#                                 MMComunications.py
#                    
#  Objective                    : Script used to login to the management port (6090)
#                                 of the MM and run dot commands to get information
#                                 about the MM and the BladeCenter chassis
#                    
#  Written by                   : Mike Nolterieke
#                                 mikenol@us.ibm.com
#                     
#  Change History               : who -   when   - what
#                                 MHN - 07/01/04 - Initial revision.
#                                 RKC - 07/26/04 - use getopt instead of optparse
#                     
###############################################################################
"""
   Script used to login to the management port (6090) of the MM and run dot commands
   to get information about the MM and the BladeCenter chassis
"""
import sys, getopt
from BladeCenterUpdates.Common import ComDefs, ComFuncs
from BladeCenterUpdates.Common.MMCommunications import MMCom
from BladeCenterUpdates.Common.Logger import Logger

# The last error that occurred
lastError   = ComDefs.SUCCESS

# The MMCom class
mmCom       = ""
logger      = ""
isTest = ComDefs.FALSE
standalone = ComDefs.FALSE
cleanExit = ComDefs.TRUE

def connectToMM(params):
    """ Connect to the MM management port (6090) and login
    """
    # Use global variable
    global mmCom
    return (mmCom.open(ComDefs.TRUE, params.mmIPAddress, 6090, 5, params.mmUserName, params.mmPassword))

def getChassisHardwareRev():
    """ Get the BladeCenter chassis hardware revision
    Returns:
        A 1 digit build ID (1, 2, 3, etc...) of the chassis hardware
        revision or -1 if there was an error
    If 0 is returned, the error code will be stored in the lastError global variable
    """
    global mmCom, lastError
    hardwareRev = -1
    # Just for my own sanity...
    # 4.3.5.5 = VPD dot command
    # Data for the command is 6 bytes:
    #   Module Type         - 1 byte
    #   Slot Number         - 1 byte
    #   Read Address LSB    - 1 byte
    #   Read Address MSB    - 1 byte
    #   Read Length LSB     - 1 byte
    #   Read Length MSB     - 1 byte
    # Module Type   = /x06  - Chassis
    # Slot Number   = /x01  - Always slot 1 for the chassis
    # Read Address  = /x0065- \x00 is MSB and \x65 is LSB
    #   These bytes must be swapped when sending the command
    #   So, \x0065 becomes \x65\x00 in LSB, MSB format
    # Read Length   = \x0002- \x00 is MSB and \x02 is LSB
    #   These bytes must also be swapped when sending the command
    #   So, \x0001 becomes \x02\x00 in LSB, MSB format
    #   Note that we only need 1 byte of information but we are getting two back
    #   This is because the first byte is ALWAYS the VPD format (IBM, OEM, etc...)
    #   We can skip that byte and go right to the data
    # Send the command and check for an error
#    lastError = mmCom.sendCmd('R', '\x04\x03\x05\x05', '\x06\x01\x65\x00\x02\x00')
    lastError = mmCom.sendCmd('R', '\x0D\x01\x01', '\x04\xA8\x28\x00\x65\x00\x02\x00')
    
    if lastError == ComDefs.SUCCESS:
        # Get the response and convert it to something useful (not hex)
        response = mmCom.getResponse()
        # The first byte [0] is the VPD format, we can skip that and go right
        # to the data [1]
        
        hardwareRev = int(ord(response[0]))
    # Return the hardware revision
    return hardwareRev

def IsRedundantMMInstalled():
    global logger, mmCom, lastError
    
    lastError = mmCom.sendCmd('R', '\x16\x02\x01', '')
    if lastError == ComDefs.SUCCESS:
        response = mmCom.getResponse()
        rMMBay = ord(response[0])
        if rMMBay > 0:
            logger.log("Redundant Management Module installed in bay %s" % rMMBay)
        return rMMBay
    else:
        logger.log("Error %s returned from command." % lastError, ComDefs.TRUE, ComDefs.TRUE)
    return ComDefs.SUCCESS

def IsMMDhcp():
    """ Returns a boolean value dependent upon whether the MM
        is configured via DHCP or not.
    """
    global logger, mmCom, lastError, params
    
    lastError = mmCom.sendCmd('R', '\x04\x09\x01\x01\x10\x02', '')

    if lastError == ComDefs.SUCCESS:
        response = mmCom.getResponse()
        
        dhcpIPaddr = "%s.%s.%s.%s" % (ord(response[0]), ord(response[1]), ord(response[2]), ord(response[3]))

        if dhcpIPaddr != "0.0.0.0":
            return ComDefs.TRUE
        else:
            return ComDefs.FALSE
    
def getBuildIDs(whichOne=1):
    """ Get the build ID's of the MM Main Application, Boot ROM, or Remote Graphics
    Values of whichOne should be 1, 2, or 3 where:
        1 = Main Application
        2 = Boot ROM
        3 = Remote Control
    Returns:
        The 7 character build ID (in text) of the specified firmware or an empty
        string if there was an error
    If an empty string is returned, the error code will be stored in the lastError
    global variable
    """
    global mmCom, lastError
    
    buildID = ""
    # Make sure they gave us a valid argument
    if ((whichOne < 1) | (whichOne > 3)):
        lastError = ComDefs.MM_INVALID_CODE_LEVEL
        return buildID
    # Convert the argument to a hex string
    whichID = ComFuncs.intToHex(whichOne, ComDefs.TRUE)

    # Send the command
    lastError = mmCom.sendCmd('R', '\x04\x03\x05\x06' + whichID, '')
    if lastError == ComDefs.SUCCESS:
        response = mmCom.getResponse()
        i = 0
        while i < len(response):
            if i < 7:
                buildID += "%s" % chr(ord(response[i]))
            else:
                pass
            i += 1
    return buildID


def getRedundantBuildIDs(whichOne=1):
    """ Get the build ID's of the MM Main Application, Boot ROM, or Remote Graphics
    Values of whichOne should be 1, 2, or 3 where:
        1 = Main Application
        2 = Boot ROM
        3 = Remote Control
    Returns:
        The 7 character build ID (in text) of the specified firmware or an empty
        string if there was an error
    If an empty string is returned, the error code will be stored in the lastError
    global variable
    """
    global mmCom, lastError
    
    buildID = ""
    # Make sure they gave us a valid argument
    if ((whichOne < 1) | (whichOne > 3)):
        lastError = ComDefs.MM_INVALID_CODE_LEVEL
        return buildID
    # Convert the argument to a hex string
    whichID = ComFuncs.intToHex(whichOne, ComDefs.TRUE)

    # Send the command
    # 22.2.4.3.5.6.x
    lastError = mmCom.sendCmd('R', '\x16\x02\x04\x03\x05\x06' + whichID, '')
    if lastError == ComDefs.SUCCESS:
        response = mmCom.getResponse()
        i = 0
        while i < len(response):
            if i < 7:
                buildID += "%s" % chr(ord(response[i]))
            else:
                pass
            i += 1
    return buildID

def usage():
    '''
        Display the Usage Information to the screen
    '''

    print "Usage: MMInfo.py [options]"
    print "\nOptions:"
    print "  -i, --ipaddress=IPADDR   The hostname or dotted IP address of the Management"
    print "                           Module"
    print "  -u, --username=USERNAME  An administrator user name used to log into the"
    print "                           Management Module"
    print "  -p, --password=PASSWORD  The password of the administrator user name used to"
    print "                           log into the Management Module"
    print "  -h, --help, -?           Display this (help) message"
    print "  -v, --verbose            Increase the debug output level by one"
    print "  -V, --version            Display the MMInfo version number and exit"


def parse_args(opts, params):
    """
       Parse the arguments and options and handle them accordingly
    """

    global isTest, cleanExit

    for o, a in opts:
        if o in ("-?", "-h", "--help"):
            usage()
            return cleanExit
        elif o in ("-v", "--verbose"):
            params.debugLevel += 1
        elif o in ("-V", "--version"):
            print ComDefs.VERSION
            return cleanExit
        # Hidden test parameter, specify -t to use our known ip address, user name,
        # and passwords to test this script
        elif o in ("-t", "--test"):
            isTest = ComDefs.TRUE
        elif o in ("-i", "--ipaddress"):
            params.mmIPAddress = a
        elif o in ("-u", "--username"):
            params.mmUserName = a
        elif o in ("-p", "--password"):
            params.mmPassword = a
        else:
            usage()
            return cleanExit

    return ComDefs.SUCCESS


def verify_args(params):
    """
       Verify that the required arguments have been supplied.
    """

    global cleanExit

    if params.mmIPAddress == "":
        print "The hostname or dotted IP address of the Management Module was not specified."
        usage()
        return cleanExit

    return ComDefs.SUCCESS


def main():
    ''' Main function used to run this script in standalone mode'''
    # Define using a global variable
    global mmCom, lastError, logger
    
    # Instanciate the class
    params = ComDefs.parameters()
    standalone = ComDefs.TRUE

    # Parse command line options with getopt()
    try:
        opts, args = getopt.getopt(sys.argv[1:], "?htvVi:u:p:",
             ["?", "help", "test", "verbose", "version", "ipaddress=", "username=", "password="])

    except getopt.GetoptError:
        # Print usage information and exit
        usage()
        sys.exit(2)

    lastError = parse_args(opts, params)
    if lastError != ComDefs.SUCCESS:
        return lastError

    # If we are in test mode (-t parameter), use our defaults and ignore the rest
    # of the parameters
    if isTest:
        # Populate the class
        params.mmIPAddress     = ""
        params.mmUserName   = ""
        params.mmPassword   = ""
        
    else:
        lastError = verify_args(params)
        if lastError != ComDefs.SUCCESS:
            return lastError

    logger = Logger(params) 
    # See if we are in debug mode
    mmCom = MMCom(params.debugLevel)
    mmCom.logger = logger
    
    # Get the MM info using the 6090 management port
    # Connect to the MM port 6090
    logger.log("Retrieve management module information.")
    logger.log("Opening connection and logging into the MM")
    if connectToMM(params) != ComDefs.SUCCESS:
        logger.log("There was an error connecting or logging into the MM!", ComDefs.TRUE, ComDefs.TRUE)
        sys.exit(1)
    else:
        logger.log("Successfully connected and logged into the MM!")

    # Get the midplane hardware revision
    logger.log("Getting chassis hardware revision...")
    hwRev = getChassisHardwareRev()
    if hwRev == "":
        logger.log("Error getting chassis hardware revision!", ComDefs.TRUE)
    else:
        logger.log("Chassis hardware revision is: %s" % hwRev, ComDefs.TRUE)
    logger.log("Getting the MM build ID's...")
    # Get the main application build ID
    buildID = getBuildIDs()
    if buildID == "":
        logger.log("Error getting Main Application build ID!", ComDefs.TRUE, ComDefs.TRUE)
    else:
        logger.log("Main Application build ID is:\t%s" % buildID, standalone)
    # Get the boot ROM build ID
    buildID = getBuildIDs(2)
    if buildID == "":
        logger.log("Error getting Boot ROM build ID!", ComDefs.TRUE, ComDefs.TRUE)
    else:
        logger.log("Boot ROM build ID is:\t\t%s" % buildID, standalone)
    # Get the remote graphics build ID
    buildID = getBuildIDs(3)
    if buildID == "":
        logger.log("Error getting Remote Control build ID!", ComDefs.TRUE, ComDefs.TRUE)
    else:
        logger.log("Remote Control build ID is:\t%s" % buildID, standalone)
    
    # JRH - Check to see if MM is configured with DHCP
    isDhcp = IsMMDhcp()
    
    if isDhcp == ComDefs.TRUE:
        logger.log("MM is configured via DHCP:\tTrue", standalone)
    else:
        logger.log("MM is configured via DHCP:\tFalse", standalone)

    # JRH - check to see if we have a redundant MM installed.
    if int(getBuildIDs(1)[4:-1]) > 51:
        rMMBay = IsRedundantMMInstalled()
        if rMMBay > 0:
            logger.log("Redundant Management Module:\tTrue", standalone)
        else:
            logger.log("Redundant Management Module:\tFalse", standalone)
    else:
        logger.log("Redundant Management Module:\tUnsupported", standalone)
        
    # Close the connection to the MM
    mmCom.close()
    
# This get's invoked if they run this script directly
if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        ComFuncs.keyboardISR(logger)

